# +++++++++++ FLASK +++++++++++
# Flask works like any other WSGI-compatible framework, we just need
# to import the application.  Often Flask apps are called "app" so we
# may need to rename it during the import:
#
#
import sys

#
## The "/home/appinv" below specifies your home
## directory -- the rest should be the directory you uploaded your Flask
## code to underneath the home directory.  So if you just ran
## "git clone git@github.com/myusername/myproject.git"
## ...or uploaded files to the directory "myproject", then you should
## specify "/home/appinv/myproject"
# on shell do pwd to get a path like this:'/home2/folder/shopyo/shopyo' set path to this
path = ""
if path not in sys.path:
    sys.path.insert(0, path)
#
from app import app as application  # noqa

#
# NB -- many Flask guides suggest you use a file called run.py; that's
# not necessary on PythonAnywhere.  And you should make sure your code
# does *not* invoke the flask development server with app.run(), as it
# will prevent your wsgi file from working.
# whatever app you specify, modify that



import re

from wtforms.validators import ValidationError

# https://wtforms.readthedocs.io/en/2.3.x/validators/


def is_empty_str(string):
    return string.strip() == ""


def is_valid_slug(text):
    # from validators package
    slug_regex = re.compile(r"^[-a-zA-Z0-9_]+$")
    return slug_regex.match(text)


def verify_slug(form, field):

    if not is_valid_slug(field.data):
        raise ValidationError(
            "Slugs can only contain alphabets, numbers and hyphens (-). eg. good-day-1"
        )


def require_if_default_address(form, field):

    if form.diffAddress.data == False:
        if is_empty_str(field.data):
            raise ValidationError("{} cannot be empty!".format(field.label))


def require_if_diff_address(form, field):
    if form.diffAddress.data == True:
        if is_empty_str(field.data):
            raise ValidationError("{} cannot be empty!".format(field.label))


def require_if_apply_coupon(form, field):

    if form.applyCoupon.data == True:
        if is_empty_str(field.data):
            raise ValidationError("{} cannot be empty!".format(field.label))


def require_if_create_account(form, field):

    if form.createAccount.data == True:
        if is_empty_str(field.data):
            raise ValidationError("{} cannot be empty!".format(field.label))



"""
Tests all db utilities such as CRUDMixin defined under shopapi/models.py
Most of the test cases taken from:
https://github.com/cookiecutter-flask/cookiecutter-flask
"""

import pytest
from flask_login import UserMixin
from shopyoapi.init import db
from shopyoapi.models import PkModel


class ExampleUserModel(PkModel, UserMixin):
    """Example user model for testing purposes"""

    __tablename__ = "testusers"

    username = db.Column(db.String(100), unique=True, nullable=False)
    email = db.Column(db.String(120), unique=True, nullable=False)


class TestPKModel:
    """Tests all functions of PKModel"""

    def test_get_by_id(self):
        result = ExampleUserModel.get_by_id("myId")

        assert not result

    def test_get_by_id_valid(self, db_session):
        user_bar = ExampleUserModel(username="bar", email="bar@domain.com")
        user_foo = ExampleUserModel(username="foo", email="foo@domain.com")
        user_bar.save()
        user_foo.save()
        result_int = ExampleUserModel.get_by_id(user_bar.id)
        result_str = ExampleUserModel.get_by_id(str(user_bar.id))
        expected = (
            db_session.query(ExampleUserModel)
            .filter(ExampleUserModel.username == "bar")
            .scalar()
        )

        assert result_int
        assert expected
        assert result_int.username == expected.username
        assert result_str.username == expected.username


class TestCRUDMixin:
    """Test class for testing all CRUD functions"""

    def test_create(self, db_session):
        user = ExampleUserModel.create(username="bar", email="bar@domain.com")
        result_raw = db_session.execute(
            """select * from testusers"""
        ).fetchone()
        result_orm = (
            db_session.query(ExampleUserModel)
            .filter(ExampleUserModel.id == user.id)
            .scalar()
        )

        assert result_orm
        assert result_raw
        assert result_raw.username == "bar"
        assert result_orm.username == "bar"

    @pytest.mark.parametrize(
        "commit,expected", [(True, "foo"), (False, "bar")]
    )
    def test_update_single(self, db_session, commit, expected):
        user = ExampleUserModel(username="bar", email="bar@domain.com")
        user.save()
        user.update(commit=commit, username="foo")
        result = db_session.execute("""select * from testusers""").fetchone()

        assert result
        assert result.username == expected

    @pytest.mark.parametrize(
        "commit,expected",
        [
            (True, {"username": "foo", "email": "foo@domain.com"}),
            (False, {"username": "bar", "email": "bar@domain.com"}),
        ],
    )
    def test_update_multiple(self, db_session, commit, expected):
        user = ExampleUserModel(username="bar", email="bar@domain.com")
        user.save()
        user.update(commit=commit, username="foo", email="foo@domain.com")
        result = db_session.execute("""select * from testusers""").fetchone()

        assert result
        assert result.username == expected["username"]
        assert result.email == expected["email"]

    @pytest.mark.parametrize("commit,expected", [(True, None), (False, "bar")])
    def test_delete(self, commit, expected):
        user = ExampleUserModel(username="bar", email="bar@domain.com")
        user.save()
        user.delete(commit=commit)
        result = ExampleUserModel.get_by_id(user.id)
        if result:
            result = result.username

        assert result == expected



from modules.box__default.settings.models import Settings


def get_setting(name):
    """
    Used as key-value lookup from Settings table

    Parameters
    ----------
    name: str
        name of key

    Returns
    -------
    str
        value of key
    """
    s = Settings.query.get(name)
    return s.value



"""
All initialisations like db = SQLAlchemy in this file
"""

from flask_login import LoginManager
from flask_marshmallow import Marshmallow
from flask_migrate import Migrate
from flask_sqlalchemy import SQLAlchemy
from flask_uploads import IMAGES
from flask_uploads import UploadSet
from flask_mailman import Mail

db = SQLAlchemy()
ma = Marshmallow()
login_manager = LoginManager()
migrate = Migrate()
mail = Mail()

productphotos = UploadSet("productphotos", IMAGES)
categoryphotos = UploadSet("categoryphotos", IMAGES)
subcategoryphotos = UploadSet("subcategoryphotos", IMAGES)



"""empty message

Revision ID: ca262f78c4b1
Revises: 
Create Date: 2021-02-24 00:17:16.505557

"""
from alembic import op
import sqlalchemy as sa


# revision identifiers, used by Alembic.
revision = "ca262f78c4b1"
down_revision = None
branch_labels = None
depends_on = None


def upgrade():
    # ### commands auto generated by Alembic - please adjust! ###
    op.create_table(
        "announcements",
        sa.Column("id", sa.Integer(), nullable=False),
        sa.Column("created_date", sa.DateTime(), nullable=True),
        sa.Column("title", sa.String(length=100), nullable=True),
        sa.Column("content", sa.String(length=1024), nullable=True),
        sa.PrimaryKeyConstraint("id"),
    )
    op.create_table(
        "appointments",
        sa.Column("id", sa.Integer(), nullable=False),
        sa.Column("name", sa.String(length=100), nullable=True),
        sa.Column("date", sa.String(length=20), nullable=True),
        sa.Column("time", sa.String(length=20), nullable=True),
        sa.Column("active", sa.String(length=20), nullable=True),
        sa.PrimaryKeyConstraint("id"),
    )
    op.create_table(
        "contact",
        sa.Column("id", sa.Integer(), nullable=False),
        sa.Column("created_date", sa.DateTime(), nullable=True),
        sa.Column("name", sa.String(length=100), nullable=True),
        sa.Column("email", sa.String(length=100), nullable=True),
        sa.Column("message", sa.String(length=1024), nullable=True),
        sa.PrimaryKeyConstraint("id"),
    )
    op.create_table(
        "images",
        sa.Column("id", sa.Integer(), nullable=False),
        sa.Column("filename", sa.String(length=50), nullable=False),
        sa.Column("thumbnail", sa.String(length=50), nullable=False),
        sa.Column("file_size", sa.Integer(), nullable=False),
        sa.Column("file_width", sa.Integer(), nullable=False),
        sa.Column("file_height", sa.Integer(), nullable=False),
        sa.Column("create_date", sa.DateTime(), nullable=False),
        sa.PrimaryKeyConstraint("id"),
    )
    op.create_table(
        "pages",
        sa.Column("id", sa.Integer(), nullable=False),
        sa.Column("created_date", sa.DateTime(), nullable=True),
        sa.Column("title", sa.String(length=100), nullable=True),
        sa.Column("slug", sa.String(length=100), nullable=True),
        sa.Column("content", sa.String(length=1024), nullable=True),
        sa.PrimaryKeyConstraint("id"),
    )
    op.create_table(
        "people",
        sa.Column("id", sa.Integer(), nullable=False),
        sa.Column("name", sa.String(length=100), nullable=True),
        sa.Column("phone", sa.Integer(), nullable=True),
        sa.Column("mobile", sa.Integer(), nullable=True),
        sa.Column("email", sa.String(length=100), nullable=True),
        sa.Column("facebook", sa.String(length=128), nullable=True),
        sa.Column("twitter", sa.String(length=128), nullable=True),
        sa.Column("linkedin", sa.String(length=128), nullable=True),
        sa.Column("age", sa.Integer(), nullable=True),
        sa.Column("birthday", sa.String(length=100), nullable=True),
        sa.Column("notes", sa.String(length=100), nullable=True),
        sa.Column("is_manufacturer", sa.Boolean(), nullable=True),
        sa.Column("manufacturer_name", sa.String(length=100), nullable=True),
        sa.Column("manufacturer_phone", sa.Integer(), nullable=True),
        sa.Column(
            "manufacturer_address", sa.String(length=200), nullable=True
        ),
        sa.PrimaryKeyConstraint("id"),
    )
    op.create_table(
        "resources",
        sa.Column("id", sa.Integer(), nullable=False),
        sa.Column("filename", sa.String(length=50), nullable=False),
        sa.Column("type", sa.String(length=50), nullable=False),
        sa.Column("category", sa.String(length=50), nullable=False),
        sa.Column("created_date", sa.DateTime(), nullable=False),
        sa.PrimaryKeyConstraint("id"),
    )
    op.create_table(
        "roles",
        sa.Column("id", sa.Integer(), nullable=False),
        sa.Column("name", sa.String(length=100), nullable=False),
        sa.PrimaryKeyConstraint("id"),
    )
    op.create_table(
        "settings",
        sa.Column("setting", sa.String(length=100), nullable=False),
        sa.Column("value", sa.String(length=100), nullable=True),
        sa.PrimaryKeyConstraint("setting"),
    )
    op.create_table(
        "users",
        sa.Column("id", sa.Integer(), nullable=False),
        sa.Column("username", sa.String(length=100), nullable=True),
        sa.Column("_password", sa.String(length=128), nullable=False),
        sa.Column("first_name", sa.String(length=128), nullable=True),
        sa.Column("last_name", sa.String(length=128), nullable=True),
        sa.Column("is_admin", sa.Boolean(), nullable=True),
        sa.Column("email", sa.String(length=120), nullable=False),
        sa.Column("date_registered", sa.DateTime(), nullable=False),
        sa.Column("is_email_confirmed", sa.Boolean(), nullable=False),
        sa.Column("email_confirm_date", sa.DateTime(), nullable=True),
        sa.PrimaryKeyConstraint("id"),
        sa.UniqueConstraint("email"),
        sa.UniqueConstraint("username"),
    )
    op.create_table(
        "role_user_bridge",
        sa.Column("user_id", sa.Integer(), nullable=False),
        sa.Column("role_id", sa.Integer(), nullable=False),
        sa.ForeignKeyConstraint(["role_id"], ["roles.id"], ondelete="CASCADE"),
        sa.ForeignKeyConstraint(["user_id"], ["users.id"], ondelete="CASCADE"),
        sa.PrimaryKeyConstraint("user_id", "role_id"),
    )
    # ### end Alembic commands ###


def downgrade():
    # ### commands auto generated by Alembic - please adjust! ###
    op.drop_table("role_user_bridge")
    op.drop_table("users")
    op.drop_table("settings")
    op.drop_table("roles")
    op.drop_table("resources")
    op.drop_table("people")
    op.drop_table("pages")
    op.drop_table("images")
    op.drop_table("contact")
    op.drop_table("appointments")
    op.drop_table("announcements")
    # ### end Alembic commands ###



"""
This file (test_auth_functional.py) contains the functional tests for
the `auth` blueprint.

These tests use GETs and POSTs to different endpoints to check
for the proper behavior of the `auth` module
"""
from flask import url_for
from flask import request
import pytest
import os
import json
import threading
from modules.box__default.auth.models import User
from flask_login import current_user

dirpath = os.path.dirname(os.path.abspath(__file__))
module_path = os.path.dirname(dirpath)

module_info = None

with open(os.path.join(module_path, "info.json")) as f:
    module_info = json.load(f)


class TestAuthInvalidAccess:
    """
    Test all auth routes for correct user authentication
    """

    routes_get = [
        "/confirm/<token>",
        "/resend",
        "/unconfirmed",
    ]

    @pytest.mark.parametrize("route", routes_get)
    def test_redirect_if_not_logged_in_get(self, test_client, route, auth):
        auth.logout()
        response = test_client.get(
            f"{module_info['url_prefix']}{route}", follow_redirects=True
        )

        assert response.status_code == 200
        assert request.path == url_for("auth.login")


class TestAuthEndpoints:
    """
    Test all auth routes' functionalities
    """

    def test_user_registration_page_renders(self, test_client):
        response = test_client.get(f"{module_info['url_prefix']}/register")

        assert response.status_code == 200
        assert b"Email" in response.data
        assert b"Password" in response.data
        assert b"Confirm Password" in response.data
        assert b"Register" in response.data

    def test_user_not_registered_on_invalid_form_submit(self, test_client):
        User.create(email="test@gmail.com", password="pass")
        data = {
            "email": "test@gmail.com",
            "password": "password",
            "confirm": "password",
        }

        response = test_client.post(
            f"{module_info['url_prefix']}/register",
            data=data,
            follow_redirects=True,
        )

        assert response.status_code == 200
        assert request.path == url_for("auth.register")

    def test_user_registration_is_case_insensitive(self, test_client):
        User.create(email="foo@bar.com", password="pass")
        data = {
            "email": "Foo@Bar.com",
            "password": "password",
            "confirm": "password",
        }

        response = test_client.post(
            f"{module_info['url_prefix']}/register",
            data=data,
            follow_redirects=True,
        )

        assert response.status_code == 200
        assert request.path == url_for("auth.register")

    @pytest.mark.parametrize(
        "email_config",
        [
            ("EMAIL_CONFIRMATION_DISABLED", True),
        ],
        indirect=True,
    )
    def test_user_confirmed_if_email_disabled(self, test_client, email_config):
        data = {
            "email": "test@gmail.com",
            "password": "password",
            "confirm": "password",
        }
        response = test_client.post(
            f"{module_info['url_prefix']}/register",
            data=data,
            follow_redirects=True,
        )
        user = User.query.filter(User.email == "test@gmail.com").scalar()

        assert response.status_code == 200
        assert request.path == url_for("dashboard.index")
        assert user.is_email_confirmed is True

    @pytest.mark.parametrize(
        "email_config",
        [
            ("EMAIL_CONFIRMATION_DISABLED", "remove"),
            ("EMAIL_CONFIRMATION_DISABLED", False),
            ("EMAIL_CONFIRMATION_DISABLED", None),
        ],
        indirect=True,
    )
    def test_user_is_registered_on_valid_form_submit(
        self, test_client, capfd, email_config
    ):
        data = {
            "email": "test@gmail.com",
            "password": "password",
            "confirm": "password",
        }
        response = test_client.post(
            f"{module_info['url_prefix']}/register",
            data=data,
            follow_redirects=True,
        )
        # Not very happy with this solution. Need a better
        # way to wait for the email thread to join with main
        # thread before reading the email written to stdout @rehmanis
        while threading.activeCount() > 1:
            pass
        else:
            captured = capfd.readouterr()

        user = User.query.filter(User.email == "test@gmail.com").scalar()

        assert response.status_code == 200
        assert request.path == url_for("auth.unconfirmed")
        assert b"A confirmation email has been sent via email" in response.data
        assert "test@gmail.com" in captured.out
        assert "Welcome to Shopyo" in captured.out
        assert user is not None
        assert user.is_email_confirmed is False

    @pytest.mark.usefixtures("login_non_admin_user")
    def test_user_not_confirmed_for_already_confirmed_user(self, test_client):
        response = test_client.get(
            url_for("auth.confirm", token="sometoken"), follow_redirects=True
        )

        assert response.status_code == 200
        assert request.path == url_for("dashboard.index")
        assert b"Account already confirmed." in response.data

    @pytest.mark.usefixtures("login_unconfirmed_user")
    def test_user_confirmed_on_valid_token(self, test_client):
        token = current_user.generate_confirmation_token()
        response = test_client.get(
            url_for("auth.confirm", token=token), follow_redirects=True
        )

        assert response.status_code == 200
        assert request.path == url_for("dashboard.index")
        assert b"You have confirmed your account. Thanks!" in response.data
        assert current_user.is_email_confirmed is True

    @pytest.mark.usefixtures("login_unconfirmed_user")
    def test_no_confirm_sent_for_invalid_token(self, test_client):
        token = current_user.generate_confirmation_token() + "extra"
        response = test_client.get(
            url_for("auth.confirm", token=token), follow_redirects=True
        )

        assert response.status_code == 200
        assert request.path == url_for("auth.unconfirmed")
        assert b"The confirmation link is invalid/expired." in response.data

    @pytest.mark.usefixtures("login_non_admin_user")
    def test_do_not_allow_email_resend_for_confirmed(self, test_client):
        response = test_client.get(
            url_for("auth.resend"), follow_redirects=True
        )

        assert response.status_code == 200
        assert request.path == url_for("dashboard.index")

    @pytest.mark.usefixtures("login_unconfirmed_user")
    def test_valid_resend_email_confirmation(self, test_client, capfd):
        response = test_client.get(
            url_for("auth.resend"), follow_redirects=True
        )

        # Not very happy with this solution. Need a better
        # way to wait for the email thread to join with main
        # thread before reading the email written to stdout @rehmanis
        while threading.activeCount() > 1:
            pass
        else:
            captured = capfd.readouterr()

        assert response.status_code == 200
        assert current_user.email in captured.out
        assert "Welcome to Shopyo" in captured.out
        assert request.path == url_for("auth.unconfirmed")
        assert b"A new confirmation email has been sent" in response.data

    @pytest.mark.usefixtures("login_non_admin_user")
    def test_confirmed_user_is_redirected_to_dashboard(self, test_client):
        response = test_client.get(
            url_for("auth.unconfirmed"), follow_redirects=True
        )

        assert response.status_code == 200
        assert request.path == url_for("dashboard.index")

    @pytest.mark.usefixtures("login_unconfirmed_user")
    def test_unconfirmed_page_renders_correctly(self, test_client):
        response = test_client.get(url_for("auth.unconfirmed"))

        assert response.status_code == 200
        assert request.path == url_for("auth.unconfirmed")
        assert b"You have not confirmed your account" in response.data
        assert b"Email confirmation link was sent to" in response.data
        assert current_user.email.encode() in response.data

    def test_login_for_dashboard_renders(self, test_client):
        response = test_client.get(url_for("auth.login"))

        response.status_code == 200
        assert b"Login" in response.data
        assert b"submit" in response.data

    def test_invalid_dashboard_login(self, test_client):
        response = test_client.post(
            url_for("auth.login"),
            data=dict(email="admin1@domain.com", password="wrongpass"),
            follow_redirects=True,
        )

        response.status_code == 200
        assert request.path == url_for("auth.login")
        assert b"please check your user id and password" in response.data

    def test_valid_dashboard_login(self, test_client, non_admin_user):
        response = test_client.post(
            url_for("auth.login"),
            data=dict(email=non_admin_user.email, password="pass"),
            follow_redirects=True,
        )

        assert response.status_code == 200
        assert current_user.email == non_admin_user.email
        assert request.path == url_for("dashboard.index")

    def test_valid_dashboard_login_is_case_insensitive(self, test_client):
        User.create(email="foo@bar.com", password="pass")
        data = {"email": "Foo@Bar.com", "password": "pass"}
        response = test_client.post(
            url_for("auth.login"),
            data=data,
            follow_redirects=True,
        )

        assert response.status_code == 200
        assert current_user.email.lower() == data["email"].lower()
        assert request.path == url_for("auth.unconfirmed")

    @pytest.mark.usefixtures("login_non_admin_user")
    def test_current_user_logout(self, test_client):
        response = test_client.get(
            url_for("auth.logout"), follow_redirects=True
        )

        assert response.status_code == 200
        assert request.path == url_for("auth.login")
        assert b"Successfully logged out" in response.data
        assert current_user.is_authenticated is False



"""
tests for all commandline arguments work as expected
Uses pytest with fixture built in fixtures
(https://docs.pytest.org/en/stable/fixture.html)
"""
import os
from shopyoapi.cmd import clean
import pytest


@pytest.mark.order("last")
class TestCommandlineClean:
    """tests the clean command line api function"""

    def test_clean_pycache_present_only_in_cwd(self, tmpdir, capfd, flask_app):
        """
        run clean on the following test directory:

        <some-unique-tmpdir>/
            __pycache__/
                file.pyc
        """
        fd = tmpdir.mkdir("__pycache__").join("file.pyc")
        fd.write("content")
        os.chdir(tmpdir)
        clean(flask_app)
        captured = capfd.readouterr()
        expected_out = (
            "[x] all tables dropped\n" "[x] __pycache__ successfully deleted\n"
        )
        expected_err_shopyo_db = (
            f"[ ] unable to delete {os.path.join(tmpdir, 'shopyo.db')}"
        )
        expected_err_migrations = (
            f"[ ] unable to delete {os.path.join(tmpdir, 'migrations')}"
        )

        assert os.path.exists("__pycache__") is False
        assert expected_out in captured.out
        assert expected_err_shopyo_db in captured.err
        assert expected_err_migrations in captured.err

    def test_clean_pycache_in_a_lvl_below_cwd(self, tmpdir, capfd, flask_app):
        """
        run clean on the following test directory:

        <some-unique-tmpdir>/
            shopyo/
                __pycache__/
                    file.pyc
        """
        shopyo_path = tmpdir.mkdir("shopyo")
        pycache_path = shopyo_path.mkdir("__pycache__")
        pyc = pycache_path.join("file.pyc")
        pyc.write("content")
        os.chdir(tmpdir)
        clean(flask_app)
        captured = capfd.readouterr()
        expected_out = (
            "[x] all tables dropped\n" "[x] __pycache__ successfully deleted\n"
        )
        expected_err_shopyo_db = (
            f"[ ] unable to delete {os.path.join(tmpdir, 'shopyo.db')}"
        )
        expected_err_migrations = (
            f"[ ] unable to delete {os.path.join(tmpdir, 'migrations')}"
        )

        assert os.path.exists(pycache_path) is False
        assert expected_out in captured.out
        assert expected_err_shopyo_db in captured.err
        assert expected_err_migrations in captured.err

    def test_clean_pycache_many_lvls_below_cwd(self, tmpdir, capfd, flask_app):
        """
        run clean on the following test directory:

        <some-unique-tmpdir>/
            shopyo/
                shopyo/
                    mod/
                        box/
                        __pycache__/
                            file.pyc
        """

        path = tmpdir.mkdir("shopyo").mkdir("shopyo").mkdir("mod").mkdir("box")
        pycache_path = path.mkdir("__pycache__")
        pyc = pycache_path.join("file.pyc")
        pyc.write("content")
        os.chdir(tmpdir)
        clean(flask_app)
        captured = capfd.readouterr()
        expected_out = (
            "[x] all tables dropped\n" "[x] __pycache__ successfully deleted\n"
        )
        expected_err_shopyo_db = (
            f"[ ] unable to delete {os.path.join(tmpdir, 'shopyo.db')}"
        )
        expected_err_migrations = (
            f"[ ] unable to delete {os.path.join(tmpdir, 'migrations')}"
        )

        assert os.path.exists(pycache_path) is False
        assert expected_out in captured.out
        assert expected_err_shopyo_db in captured.err
        assert expected_err_migrations in captured.err

    def test_clean_many_pycache_in_nested_dirs(self, tmpdir, capfd, flask_app):
        """
        run clean on the following test directory:

        <some-unique-tmpdir>/
            __pycache__/
                file.pyc
            shopyo/
                __pycache__
                    file.pyc
                module/
                    __pycache__
                        file.pyc
        """
        pycache_path1 = tmpdir.mkdir("__pycache__")
        pyc1 = pycache_path1.join("file.pyc")
        pyc1.write("content")
        shopyo_path = tmpdir.mkdir("shopyo")
        pycache_path2 = shopyo_path.mkdir("__pycache__")
        pyc2 = pycache_path2.join("file.pyc")
        pyc2.write("content")
        pycache_path3 = shopyo_path.mkdir("module").mkdir("__pycache__")
        pyc3 = pycache_path3.join("file.pyc")
        pyc3.write("content")
        os.chdir(tmpdir)
        clean(flask_app)
        captured = capfd.readouterr()
        expected_out = (
            "[x] all tables dropped\n" "[x] __pycache__ successfully deleted\n"
        )
        expected_err_shopyo_db = (
            f"[ ] unable to delete {os.path.join(tmpdir, 'shopyo.db')}"
        )
        expected_err_migrations = (
            f"[ ] unable to delete {os.path.join(tmpdir, 'migrations')}"
        )

        assert os.path.exists(pycache_path1) is False
        assert os.path.exists(pycache_path2) is False
        assert os.path.exists(pycache_path3) is False
        assert expected_out in captured.out
        assert expected_err_shopyo_db in captured.err
        assert expected_err_migrations in captured.err

    def test_no_clean_applied_on_multiple_pycache(self, tmpdir, capfd):
        """
        run clean on the following test directory:

        <some-unique-tmpdir>/
                __pycache__/
                shopyo/
                    __pycache__/

        """
        path1 = tmpdir.mkdir("__pycache__")
        path2 = tmpdir.mkdir("shopyo").mkdir("__pycache__")

        assert os.path.exists(path1)
        assert os.path.exists(path2)

    def test_clean_on_shopyo_db_file(self, tmpdir, capfd, flask_app):
        """
        run clean on the following test directory:

        <some-unique-tmpdir>/
            shopyo.db
        """
        shopyo_db = tmpdir.join("shopyo.db")
        shopyo_db.write("content")
        os.chdir(tmpdir)
        clean(flask_app)
        captured = capfd.readouterr()
        expected_out = (
            "[x] all tables dropped\n"
            f"[x] file '{os.path.join(tmpdir, 'shopyo.db')}' "
            "successfully deleted\n"
        )
        expected_err_pycache = "[ ] __pycache__ doesn't exist\n"
        expected_err_migrations = (
            f"[ ] unable to delete {os.path.join(tmpdir, 'migrations')}"
        )

        assert os.path.exists(shopyo_db) is False
        assert expected_out in captured.out
        assert expected_err_pycache in captured.err
        assert expected_err_migrations in captured.err

    def test_clean_on_migration_folder(self, tmpdir, capfd, flask_app):
        """
        run clean on the following test directory:

        <some-unique-tmpdir>/
            migrations/
                env.py
                alembic.ini
        """
        migrations_path = tmpdir.mkdir("migrations")
        env = migrations_path.join("env.py")
        alembic = migrations_path.join("alembic.ini")
        env.write("content-env")
        alembic.write("content-alembic")
        os.chdir(tmpdir)
        clean(flask_app)
        captured = capfd.readouterr()
        expected_out = (
            "[x] all tables dropped\n"
            f"[x] folder '{os.path.join(tmpdir, 'migrations')}' "
            "successfully deleted\n"
        )
        expected_err_pycache = "[ ] __pycache__ doesn't exist\n"
        expected_err_shopyo_db = (
            f"[ ] unable to delete {os.path.join(tmpdir, 'shopyo.db')}"
        )

        assert os.path.exists(migrations_path) is False
        assert expected_out in captured.out
        assert expected_err_pycache in captured.err
        assert expected_err_shopyo_db in captured.err

    def test_clean_on_pycache_shopyo_migration(self, tmpdir, flask_app, capfd):
        """
        run clean on the following test directory

        shopyo/
            shopyo/
                migrations/
                    alembic.ini
                    env.py
                module1/
                    __pycache__/
                        file.pyc
                module2/
                    __pycache__/
                        file.pyc
                shopyo.db
        """
        shopyo_path = tmpdir.mkdir("shopyo").mkdir("shopyo")
        migrations_path = shopyo_path.mkdir("migrations")
        env = migrations_path.join("env.py")
        env.write("content-env")
        alembic = migrations_path.join("alembic.ini")
        alembic.write("content-alembic")
        pycache_path1 = shopyo_path.mkdir("module1").mkdir("__pycache__")
        pycache_path2 = shopyo_path.mkdir("module2").mkdir("__pycache__")
        pyc1 = pycache_path1.join("file.pyc")
        pyc1.write("content")
        pyc2 = pycache_path2.join("file.pyc")
        pyc2.write("content")
        shopyo_db = shopyo_path.join("shopyo.db")
        shopyo_db.write("content")
        os.chdir(shopyo_path)
        clean(flask_app)
        captured = capfd.readouterr()
        expected_out = (
            "[x] all tables dropped\n"
            "[x] __pycache__ successfully deleted\n"
            f"[x] file '{os.path.join(shopyo_path, 'shopyo.db')}' "
            "successfully deleted\n"
            f"[x] folder '{os.path.join(shopyo_path, 'migrations')}' "
            "successfully deleted\n"
        )

        assert expected_out in captured.out
        assert os.path.exists(migrations_path) is False
        assert os.path.exists(pycache_path1) is False
        assert os.path.exists(pycache_path2) is False
        assert os.path.exists(shopyo_db) is False

    def test_no_clean_on_shopyo_and_migrations(self, tmpdir):
        """
        run test on the following test directory:

        <some-unique-tmpdir>/
            migrations/
            shopyo.db
        """
        migrations_path = tmpdir.mkdir("migrations")
        shopyo_db = tmpdir.join("shopyo.db")
        shopyo_db.write("content")

        assert os.path.exists(migrations_path)
        assert os.path.exists(shopyo_db)

    def test_clean_on_no_files_to_clean(self, tmpdir, capfd, flask_app):
        os.chdir(tmpdir)
        clean(flask_app)
        captured = capfd.readouterr()
        expected_out = "[x] all tables dropped\n"
        expected_err_pycache = "[ ] __pycache__ doesn't exist\n"
        expected_err_shopyo_db = (
            f"[ ] unable to delete {os.path.join(tmpdir, 'shopyo.db')}"
        )
        expected_err_migrations = (
            f"[ ] unable to delete {os.path.join(tmpdir, 'migrations')}"
        )

        assert expected_out in captured.out
        assert expected_err_pycache in captured.err
        assert expected_err_shopyo_db in captured.err
        assert expected_err_migrations in captured.err

    # TODO: add test_clean for postgresSQL to see if tables dropped @rehmanis



from flask_wtf import FlaskForm

# from wtforms.validators import Length
from wtforms import StringField
from wtforms import TextAreaField
from wtforms.fields.html5 import EmailField
from wtforms.validators import DataRequired


class ContactForm(FlaskForm):
    name = StringField(
        "Name",
        [DataRequired()],
        render_kw={"class": "form-control", "autocomplete": "off"},
    )
    email = EmailField(
        "Email",
        [DataRequired()],
        render_kw={"class": "form-control", "autocomplete": "off"},
    )
    message = TextAreaField(
        "Message",
        [DataRequired()],
        render_kw={
            "class": "form-control",
            "rows": "20",
            "autocomplete": "off",
        },
    )



import json
import os

# from flask import render_template
from flask import Blueprint
from flask import current_app

# from flask import redirect
from flask import send_from_directory

# from flask import url_for

from flask_login import login_required
from PIL import Image as PILimage

# from shopyoapi.enhance import get_setting
# from shopyoapi.file import delete_file

# from modules.box__ecommerce.product.models import Product
from modules.resource.models import Image

# from modules.resource.models import Resource

# from flask import flash
# from flask import request#
# from shopyoapi.html import notify_success
# from shopyoapi.forms import flash_errors


dirpath = os.path.dirname(os.path.abspath(__file__))
module_info = {}

with open(dirpath + "/info.json") as f:
    module_info = json.load(f)

globals()["{}_blueprint".format(module_info["module_name"])] = Blueprint(
    "{}".format(module_info["module_name"]),
    __name__,
    template_folder="templates",
    url_prefix=module_info["url_prefix"],
)


module_blueprint = globals()["{}_blueprint".format(module_info["module_name"])]


@module_blueprint.route("/")
def index():
    return module_info["display_string"]


@module_blueprint.route(
    "/theme/front/<active_theme>/styles.css", methods=["GET"]
)
def active_front_theme_css(active_theme):
    theme_dir = os.path.join(
        current_app.config["BASE_DIR"],
        "static",
        "themes",
        "front",
        active_theme,
    )
    # return theme_dir
    return send_from_directory(theme_dir, "styles.css")


@module_blueprint.route(
    "/theme/back/<active_theme>/styles.css", methods=["GET"]
)
def active_back_theme_css(active_theme):
    theme_dir = os.path.join(
        current_app.config["BASE_DIR"],
        "static",
        "themes",
        "back",
        active_theme,
    )
    # return theme_dir
    return send_from_directory(theme_dir, "styles.css")


# Handles javascript image uploads from tinyMCE
@module_blueprint.route("/upload/tinymce/image", methods=["POST"])
@login_required
def upload_tinymce_image():
    # Kevin Foong
    file = request.files.get("file")
    if file:
        filename = file.filename.lower()
        fn, ext = filename.split(".")
        # truncate filename (excluding extension) to 30 characters
        fn = fn[:30]
        filename = fn + "." + ext
        if ext in ["jpg", "gif", "png", "jpeg"]:
            try:
                # everything looks good, save file
                img_fullpath = os.path.join(
                    current_app.config["UPLOADED_PATH_IMAGE"], filename
                )
                file.save(img_fullpath)
                # get the file size to save to db
                file_size = os.stat(img_fullpath).st_size
                size = 160, 160
                # read image into pillow
                im = PILimage.open(img_fullpath)
                # get image dimension to save to db
                file_width, file_height = im.size
                # convert to thumbnail
                im.thumbnail(size)
                thumbnail = fn + "-thumb.jpg"
                tmb_fullpath = os.path.join(
                    current_app.config["UPLOADED_PATH_THUMB"], thumbnail
                )
                # PNG is index while JPG needs RGB
                if not im.mode == "RGB":
                    im = im.convert("RGB")
                # save thumbnail
                im.save(tmb_fullpath, "JPEG")

                # save to db
                img = Image(
                    filename=filename,
                    thumbnail=thumbnail,
                    file_size=file_size,
                    file_width=file_width,
                    file_height=file_height,
                )
                db.session.add(img)
                db.session.commit()
            except IOError:
                output = make_response(404)
                output.headers["Error"] = (
                    "Cannot create thumbnail for " + filename
                )
                return output
            return jsonify({"location": filename})

    # fail, image did not upload
    output = make_response(404)
    output.headers["Error"] = "Filename needs to be JPG, JPEG, GIF or PNG"
    return output



import json
import os

from flask import Blueprint
from flask import redirect
from flask import render_template
from flask import request
from flask import url_for

from flask_login import login_required

from shopyoapi.forms import flash_errors

from .forms import PageForm
from .models import Page

dirpath = os.path.dirname(os.path.abspath(__file__))
module_info = {}

with open(dirpath + "/info.json") as f:
    module_info = json.load(f)

globals()["{}_blueprint".format(module_info["module_name"])] = Blueprint(
    "{}".format(module_info["module_name"]),
    __name__,
    template_folder="templates",
    url_prefix=module_info["url_prefix"],
)


module_blueprint = globals()["{}_blueprint".format(module_info["module_name"])]

module_name = module_info["module_name"]

sidebar = [{"text": "sample", "icon": "fa fa-table", "url": ""}]

module_settings = {"sidebar": sidebar}


@module_blueprint.route("/")
def index():
    context = {}
    pages = Page.query.all()

    context.update({"pages": pages})
    return render_template("page/all_pages.html", **context)


@module_blueprint.route("/<page_id>/<slug>")
def view_page(page_id, slug):
    context = {}
    page = Page.query.get(page_id)

    context.update({"page": page})
    return render_template("page/view_page.html", **context)


@module_blueprint.route(module_info["dashboard"])
@login_required
def dashboard():
    context = {}
    form = PageForm()

    context.update({"form": form, "module_name": module_name})
    context.update(module_settings)
    return render_template("page/dashboard.html", **context)


@module_blueprint.route("/check_pagecontent", methods=["GET", "POST"])
@login_required
def check_pagecontent():
    if request.method == "POST":
        form = PageForm()
        if not form.validate_on_submit():
            flash_errors(form)
            return redirect(url_for("{}.dashboard".format(module_name)))
        toaddpage = Page(
            slug=form.slug.data,
            content=form.content.data,
            title=form.title.data,
        )
        toaddpage.insert()
        return redirect(url_for("{}.dashboard".format(module_name)))



from flask import url_for
from flask import current_app


def get_static(boxormodule, filename):
    if current_app.config["DEBUG"] == True:
        return url_for("devstatic", boxormodule=boxormodule, filename=filename)
    else:
        return url_for(
            "static", filename="modules/{}/{}".format(boxormodule, filename)
        )



from shopyoapi.init import db


class Appointments(db.Model):
    __tablename__ = "appointments"
    id = db.Column(db.Integer, primary_key=True)
    name = db.Column(db.String(100))
    date = db.Column(db.String(20))
    time = db.Column(db.String(20))
    active = db.Column(db.String(20))
